import datamazing.pandas as pdz
import pandas as pd
from typeguard import typechecked


class ResourceManager:
    """
    Manager which simplifies the process of handling resource schedules.
    """

    def __init__(
        self,
        db: pdz.Database,
        time_interval: pdz.TimeInterval,
    ) -> None:
        self.db = db
        self.time_interval = time_interval

    @typechecked
    def get_resource_schedules(
        self,
        resource_gsrns: list,
    ) -> pd.DataFrame:
        """Gets resource schedules for a given list of resource gsrns."""
        df_resource_schedules = self.db.query(
            table_name="resourceSchedule",
            time_interval=self.time_interval,
            filters={"resource_gsrn": resource_gsrns},
        )
        df_resource_schedules = df_resource_schedules.filter(
            [
                "market_participant",
                "created_time_utc",
                "price_area",
                "resource_gsrn",
                "main_fuel_type",
                "is_sum_plan",
                "time_utc",
                "schedule_power_MW",
                "schedule_capacity_min_MW",
                "schedule_capacity_max_MW",
            ]
        )
        return df_resource_schedules

    @typechecked
    def get_latest_resource_plan(
        self,
        resource_gsrns: list,
    ) -> pd.DataFrame:
        """Gets the lastest resource schedules for a given list of resource gsrns."""

        df_resource_schedules = self.get_resource_schedules(
            resource_gsrns=resource_gsrns
        )

        df_latest_created_time = pdz.group(
            df=df_resource_schedules, by=["resource_gsrn", "time_utc"]
        ).agg({"created_time_utc": max})

        df_resource_latest = df_latest_created_time.merge(
            df_resource_schedules, on=list(df_latest_created_time.columns)
        )

        return df_resource_latest



import datamazing.pandas as pdz
import pandas as pd

from .masterdata_manager import MasterdataManager


class PlantManager(MasterdataManager):
    """
    Manager which simplifies the process of getting plants from masterdata.
    """

    def __init__(
        self,
        db: pdz.Database,
        time_interval: pdz.TimeInterval,
        resolution: pd.Timedelta,
    ) -> None:
        self.db = db
        self.time_interval = time_interval
        self.resolution = resolution

    def get_plants(
        self,
        filters: dict = {},
        columns: list = [
            "plant_id",
            "masterdata_gsrn",
            "datahub_gsrn_e18",
            "installed_power_MW",
            "price_area",
            "valid_from_date_utc",
            "valid_to_date_utc",
        ],
    ) -> pd.DataFrame:
        """Gets the plants for a given plant type.
        Filters for plants valid at the end of time interval.
        Filters by default for plants in operation.
        """
        return self.get_data("masterdataPlant", filters=filters, columns=columns)

    def get_installed_power_timeseries(self, gsrn: str) -> pd.DataFrame:
        """Gets the installed power timeseries for a plant."""

        df_times = self.time_interval.to_range(self.resolution).to_frame(
            index=False, name="time_utc"
        )

        # explode plant to time series
        df_plant = self.get_operational_entities("masterdataPlant")
        df_plant = df_plant.query(f"masterdata_gsrn == '{gsrn}'")

        df_plant = pdz.merge(
            df_times,
            df_plant,
            left_time="time_utc",
            right_period=("valid_from_date_utc", "valid_to_date_utc"),
        )

        return df_plant.filter(["time_utc", "installed_power_MW"]).reset_index(
            drop=True
        )
